/*
 * Fahrregler H-Bridge
 * Januar 2014, Uli Beyenbach
 * Pin6 EN (PWM)
 * Pin4 IN1
 * Pin5 IN2
 * Pin3 NotHalt (LOW=aktiv)
 * Pin7 LED LI
 * Pin8 LED RE
 * Pin9 LED STOPP
 * Pin10 LED NOTHALT
 *
 * PinA0 Fahrregler
 * PinA1 Massensimulation
 * PinA2 Offset
 * 
*/
 
//
const byte EN=6;
const byte IN1=4;
const byte IN2=5;
const byte NotHalt=3;
const byte LI=7;
const byte RE=8;
const byte STOPP=9;
const byte NOTHALT=10;

int soll=0;
byte ist=0;
byte richtung=0;  //1=li, 2=re, 0=halt
byte richtungsoll=0;
byte haltflag=0;
int regler;
int masse;
int offset;
int pwm;
volatile unsigned int tick;
volatile byte tcnt2;


byte v[64]={  0,  1,  2,  3,  4,  5,  6,  7,  8, 10, 12,
                 14, 16, 18, 20, 22, 24, 26, 28, 30, 32,
                 35, 38, 41, 45, 48, 51, 54, 58, 62, 66,
                 70, 74, 78, 82, 86, 91, 96,101,107,113,
                119,125,131,137,143,149,155,161,167,173,
                179,185,191,197,203,209,215,221,227,234,
                241,248,255};
             
//--------------------------------------------------------------------
ISR(TIMER2_OVF_vect) {  
  /* Reload the timer */  
  TCNT2 = tcnt2;  
  tick++;
} 
//--------------------------------------------------------------------
 
void setup() {
  //TCCR0B = TCCR0B & B11111000 | B00000001;    // set timer 0 divisor to     1 for PWM frequency of 62500.00 Hz
  TCCR0B = TCCR0B & B11111000 | B00000010;    // set timer 0 divisor to     8 for PWM frequency of  7812.50 Hz
  //Wird obige Zeile auskommentiert, gilt die Standardfreuqenz von 976,56 Hz 
  pinMode(IN1, OUTPUT);
  pinMode(IN2, OUTPUT);
  pinMode(NotHalt, INPUT_PULLUP);
  pinMode(LI, OUTPUT);
  pinMode(RE, OUTPUT);
  pinMode(STOPP, OUTPUT);
  pinMode(NOTHALT, OUTPUT);
  /* Timer2 wird verändert, somit ist PWM an Pin3 und Pin11 nicht möglich 
     Timer2 zählt "tick" im 0,5ms Rhythmus hoch.
  */
      /* First disable the timer overflow interrupt while we're configuring */  
      TIMSK2 &= ~(1<<TOIE2);  
      
      /* Configure timer2 in normal mode (pure counting, no PWM etc.) */  
      TCCR2A &= ~((1<<WGM21) | (1<<WGM20));  
      TCCR2B &= ~(1<<WGM22);  
      
      /* Disable Compare Match A interrupt enable (only want overflow) */  
      TIMSK2 &= ~(1<<OCIE2A);  
      
      /* Now configure the prescaler to CPU clock divided by 128 */  
      TCCR2B |= (1<<CS22)  | (1<<CS20); // Set bits  
      TCCR2B &= ~(1<<CS21);             // Clear bit  
      
      /* We need to calculate a proper value to load the timer counter. 
       * The following loads the value 194 into the Timer 2 counter register 
       * The math behind this is: 
       * (CPU frequency = 16MHz) / (prescaler value) = 125000 Hz = 8us. 
       * (desired period = 0,5ms) / 8us = 62.
       * MAX(uint8) + 1 - 62 = 194.
       */  
      /* Save value globally for later reload in ISR */  
      tcnt2 = 194;   // 0,5 ms
      
      /* Finally load end enable the timer */  
      TCNT2 = tcnt2;  
      TIMSK2 |= (1<<TOIE2);    
}

void loop() {
  regler=analogRead(A0);  // 0 bis 1023, 0-500 LI, 501-522, STOPP, 523-1023 RE
  masse=analogRead(A1)/4; // Massensimulation
  offset=analogRead(A2)/4;  //zwischen 0 und 255
                            //
                            //soll liegt zwischen 0 und 500 und wird in "sollIst" auf 0 bis 63 umgerechnet
  if(regler<501){
    digitalWrite(LI, HIGH);
    digitalWrite(RE, LOW);
    digitalWrite(STOPP, LOW);
    soll=500-regler;
    richtungsoll=1;
  }
  if(regler>500 && regler<523){
    digitalWrite(LI, LOW);
    digitalWrite(RE, LOW);
    digitalWrite(STOPP, HIGH);
    soll=0;
    richtungsoll=0;
    if(haltflag){
      haltflag=0;
      ist=0;
    }
  }
  if(regler>522){
    digitalWrite(LI, LOW);
    digitalWrite(RE, HIGH);
    digitalWrite(STOPP, LOW);
    soll=regler-523;
    richtungsoll=2;
  }
  if(digitalRead(NotHalt)==LOW) haltflag=1;
  
  if(haltflag){
    digitalWrite(IN1,LOW);
    digitalWrite(IN2,LOW);
    digitalWrite(NOTHALT,HIGH);
    ist=63; //????????????????????????????????????????????????? =0 ???????????????????????
  }
  else{
    digitalWrite(NOTHALT,LOW);
    sollIst();
  }
  pwm=v[ist]+offset;
  if(pwm>255) pwm=255;
  if(pwm<0 || ist==0) pwm=0;
    analogWrite(EN,pwm);
}

void sollIst(){
  soll=map(soll,0,500,0,63);
  if(richtung != richtungsoll){
    if(ist>0){
      langsamer();
      return;
    }
    else{
      richtung=richtungsoll;
      switch(richtung){
        case 1:
        digitalWrite(IN1,HIGH);
        digitalWrite(IN2,LOW);
        break;
        case 2:
        digitalWrite(IN1,LOW);
        digitalWrite(IN2,HIGH);
        break;
      }
    }
  }
  if(soll>ist) schneller();
  if(soll<ist) langsamer();
}

void langsamer(){
  if(tick>masse){
    tick=1;
    if(ist) ist--;
  }
}

void schneller(){
  if(tick>masse){
    tick=1;
    if(ist<63) ist++;
  }
}

